/**
 * Multi-Provider AI System for ZAI MCP Server
 * Supports OpenRouter, Anthropic, and DeepSeek APIs with automatic failover
 * Updated with current OpenRouter models (2025)
 */

import fetch from 'node-fetch';

export class MultiProviderAI {
    constructor() {
        this.providers = {
            openrouter: {
                name: 'OpenRouter',
                enabled: false,
                apiKeys: [],
                currentKeyIndex: 0,
                failedKeys: new Set(),
                baseUrl: 'https://openrouter.ai/api/v1',
                models: [
                    // Free Models (2025)
                    'deepseek/deepseek-r1-0528:free',
                    'deepseek/deepseek-r1-0528-qwen3-8b:free',
                    'mistralai/devstral-small:free',
                    'sarvamai/sarvam-m:free',
                    'google/gemma-3n-e4b-it:free',
                    'meta-llama/llama-3.3-8b-instruct:free',
                    'microsoft/phi-4-reasoning:free',
                    'microsoft/phi-4-reasoning-plus:free',
                    'qwen/qwen3-8b:free',
                    'qwen/qwen3-14b:free',
                    'qwen/qwen3-30b-a3b:free',
                    'qwen/qwen3-32b:free',
                    'thudm/glm-z1-32b:free',
                    
                    // Popular Paid Models
                    'anthropic/claude-opus-4',
                    'anthropic/claude-sonnet-4',
                    'google/gemini-2.5-pro-preview',
                    'google/gemini-2.5-flash-preview-05-20',
                    'openai/gpt-4o',
                    'openai/gpt-4o-mini',
                    'openai/o1-preview',
                    'openai/o1-mini',
                    'anthropic/claude-3-5-sonnet-20241022',
                    'anthropic/claude-3-5-haiku-20241022',
                    'deepseek/deepseek-chat',
                    'deepseek/deepseek-reasoner'
                ],
                defaultModel: 'deepseek/deepseek-r1-0528:free'
            },
            anthropic: {
                name: 'Anthropic',
                enabled: false,
                apiKeys: [],
                currentKeyIndex: 0,
                failedKeys: new Set(),
                baseUrl: 'https://api.anthropic.com/v1',
                models: [
                    'claude-opus-4-20250514',
                    'claude-sonnet-4-20250514',
                    'claude-3-5-sonnet-20241022',
                    'claude-3-5-haiku-20241022'
                ],
                defaultModel: 'claude-3-5-sonnet-20241022'
            },
            deepseek: {
                name: 'DeepSeek',
                enabled: false,
                apiKeys: [],
                currentKeyIndex: 0,
                failedKeys: new Set(),
                baseUrl: 'https://api.deepseek.com/v1',
                models: [
                    'deepseek-chat',
                    'deepseek-reasoner'
                ],
                defaultModel: 'deepseek-chat'
            },
            gemini: {
                name: 'Google Gemini',
                enabled: false,
                apiKeys: [],
                currentKeyIndex: 0,
                failedKeys: new Set(),
                baseUrl: 'https://generativelanguage.googleapis.com/v1beta',
                models: [
                    // 2025 Latest Models (Confirmed Working)
                    'gemini-2.5-flash-preview-05-20',
                    'gemini-2.0-flash',
                    'gemini-2.0-flash-001',
                    'gemini-2.0-flash-lite',
                    'gemini-1.5-flash-latest',
                    'gemini-1.5-flash-8b-latest',
                    'gemini-1.5-flash',
                    'gemini-1.5-flash-8b',
                    // Experimental Models
                    'gemini-2.0-flash-exp',
                    'gemini-2.5-pro-preview-06-05',
                    'gemini-2.0-flash-thinking-exp-01-21'
                ],
                defaultModel: 'gemini-2.5-flash-preview-05-20'
            }
        };

        this.currentProvider = null;
        this.requestCount = 0;
        this.successCount = 0;
        this.errorCount = 0;

        this.initializeProviders();
    }

    initializeProviders() {
        // Initialize OpenRouter
        const openrouterKeys = process.env.OPENROUTER_API_KEY?.split(',').map(key => key.trim()).filter(Boolean) || [];
        if (openrouterKeys.length > 0) {
            this.providers.openrouter.apiKeys = openrouterKeys;
            this.providers.openrouter.enabled = true;
            console.log(`✅ OpenRouter initialized with ${openrouterKeys.length} API keys`);
        }

        // Initialize Anthropic
        const anthropicKeys = process.env.ANTHROPIC_API_KEY?.split(',').map(key => key.trim()).filter(Boolean) || [];
        if (anthropicKeys.length > 0) {
            this.providers.anthropic.apiKeys = anthropicKeys;
            this.providers.anthropic.enabled = true;
            console.log(`✅ Anthropic initialized with ${anthropicKeys.length} API keys`);
        }

        // Initialize DeepSeek
        const deepseekKeys = process.env.DEEPSEEK_API_KEY?.split(',').map(key => key.trim()).filter(Boolean) || [];
        if (deepseekKeys.length > 0) {
            this.providers.deepseek.apiKeys = deepseekKeys;
            this.providers.deepseek.enabled = true;
            console.log(`✅ DeepSeek initialized with ${deepseekKeys.length} API keys`);
        }

        // Initialize Google Gemini
        const geminiKeys = process.env.GEMINI_API_KEY?.split(',').map(key => key.trim()).filter(Boolean) || [];
        if (geminiKeys.length > 0) {
            this.providers.gemini.apiKeys = geminiKeys;
            this.providers.gemini.enabled = true;
            console.log(`✅ Google Gemini initialized with ${geminiKeys.length} API keys`);
        }

        // Set current provider (prioritize OpenRouter for free models)
        this.currentProvider = this.providers.openrouter.enabled ? 'openrouter' :
                              this.providers.gemini.enabled ? 'gemini' :
                              this.providers.deepseek.enabled ? 'deepseek' :
                              this.providers.anthropic.enabled ? 'anthropic' : null;

        if (!this.currentProvider) {
            console.warn('⚠️ No AI providers configured. Please set API keys.');
        } else {
            console.log(`🎯 Primary provider: ${this.providers[this.currentProvider].name}`);
        }
    }

    async makeRequest(prompt, options = {}) {
        const {
            maxTokens = 1000,
            temperature = 0.7,
            model = null,
            provider = null
        } = options;

        this.requestCount++;
        const startTime = Date.now();

        // Try specified provider first, then fallback
        const providersToTry = provider ? [provider] : this.getProviderPriority();

        for (const providerName of providersToTry) {
            const providerConfig = this.providers[providerName];
            
            if (!providerConfig.enabled) continue;

            try {
                const response = await this.makeProviderRequest(
                    providerName,
                    prompt,
                    {
                        maxTokens,
                        temperature,
                        model: model || providerConfig.defaultModel
                    }
                );

                this.successCount++;
                const responseTime = Date.now() - startTime;

                console.log(`✅ AI request successful: ${providerName}/${response.model} (${responseTime}ms)`);

                return {
                    content: response.content,
                    provider: providerName,
                    model: response.model,
                    responseTime,
                    usage: response.usage
                };

            } catch (error) {
                console.warn(`⚠️ Provider ${providerName} failed: ${error.message}`);
                this.handleProviderError(providerName, error);
                continue;
            }
        }

        this.errorCount++;
        throw new Error('All AI providers failed. Please check your API keys and try again.');
    }

    async makeProviderRequest(providerName, prompt, options) {
        const provider = this.providers[providerName];
        const apiKey = this.getNextApiKey(providerName);
        
        if (!apiKey) {
            throw new Error(`No available API keys for ${providerName}`);
        }

        const { maxTokens, temperature, model } = options;

        switch (providerName) {
            case 'openrouter':
                return await this.makeOpenRouterRequest(apiKey, prompt, { maxTokens, temperature, model });
            
            case 'anthropic':
                return await this.makeAnthropicRequest(apiKey, prompt, { maxTokens, temperature, model });
            
            case 'deepseek':
                return await this.makeDeepSeekRequest(apiKey, prompt, { maxTokens, temperature, model });

            case 'gemini':
                return await this.makeGeminiRequest(apiKey, prompt, { maxTokens, temperature, model });

            default:
                throw new Error(`Unknown provider: ${providerName}`);
        }
    }

    async makeOpenRouterRequest(apiKey, prompt, options) {
        const { maxTokens, temperature, model } = options;
        
        const response = await fetch(`${this.providers.openrouter.baseUrl}/chat/completions`, {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${apiKey}`,
                'Content-Type': 'application/json',
                'HTTP-Referer': 'https://github.com/Zrald1/zai-mcp-server',
                'X-Title': 'ZAI MCP Server'
            },
            body: JSON.stringify({
                model,
                messages: [
                    {
                        role: 'user',
                        content: prompt
                    }
                ],
                max_tokens: maxTokens,
                temperature,
                stream: false
            })
        });

        if (!response.ok) {
            const error = await response.text();
            throw new Error(`OpenRouter API error: ${response.status} - ${error}`);
        }

        const data = await response.json();
        
        return {
            content: data.choices[0].message.content,
            model: data.model,
            usage: data.usage
        };
    }

    async makeAnthropicRequest(apiKey, prompt, options) {
        const { maxTokens, temperature, model } = options;
        
        const response = await fetch(`${this.providers.anthropic.baseUrl}/messages`, {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${apiKey}`,
                'Content-Type': 'application/json',
                'anthropic-version': '2023-06-01'
            },
            body: JSON.stringify({
                model,
                max_tokens: maxTokens,
                temperature,
                messages: [
                    {
                        role: 'user',
                        content: prompt
                    }
                ]
            })
        });

        if (!response.ok) {
            const error = await response.text();
            throw new Error(`Anthropic API error: ${response.status} - ${error}`);
        }

        const data = await response.json();
        
        return {
            content: data.content[0].text,
            model: data.model,
            usage: data.usage
        };
    }

    async makeDeepSeekRequest(apiKey, prompt, options) {
        const { maxTokens, temperature, model } = options;

        const response = await fetch(`${this.providers.deepseek.baseUrl}/chat/completions`, {
            method: 'POST',
            headers: {
                'Authorization': `Bearer ${apiKey}`,
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                model,
                messages: [
                    {
                        role: 'user',
                        content: prompt
                    }
                ],
                max_tokens: maxTokens,
                temperature,
                stream: false
            })
        });

        if (!response.ok) {
            const error = await response.text();
            throw new Error(`DeepSeek API error: ${response.status} - ${error}`);
        }

        const data = await response.json();

        return {
            content: data.choices[0].message.content,
            model: data.model,
            usage: data.usage
        };
    }

    async makeGeminiRequest(apiKey, prompt, options) {
        const { maxTokens, temperature, model } = options;

        const response = await fetch(`${this.providers.gemini.baseUrl}/models/${model}:generateContent?key=${apiKey}`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                contents: [
                    {
                        parts: [
                            {
                                text: prompt
                            }
                        ]
                    }
                ],
                generationConfig: {
                    maxOutputTokens: maxTokens,
                    temperature: temperature,
                    topP: 0.8,
                    topK: 10
                }
            })
        });

        if (!response.ok) {
            const error = await response.text();
            throw new Error(`Gemini API error: ${response.status} - ${error}`);
        }

        const data = await response.json();

        if (!data.candidates || !data.candidates[0]) {
            throw new Error('Gemini API returned no candidates');
        }

        const candidate = data.candidates[0];

        if (!candidate.content || !candidate.content.parts || !candidate.content.parts[0]) {
            throw new Error('Gemini API returned no content parts');
        }

        return {
            content: candidate.content.parts[0].text,
            model: model,
            usage: data.usageMetadata || {}
        };
    }

    getNextApiKey(providerName) {
        const provider = this.providers[providerName];
        const availableKeys = provider.apiKeys.filter((_, index) => 
            !provider.failedKeys.has(index)
        );

        if (availableKeys.length === 0) {
            return null;
        }

        const key = provider.apiKeys[provider.currentKeyIndex];
        provider.currentKeyIndex = (provider.currentKeyIndex + 1) % provider.apiKeys.length;
        
        return key;
    }

    handleProviderError(providerName, error) {
        const provider = this.providers[providerName];
        
        // Mark current key as failed if it's a key-specific error
        if (error.message.includes('401') || error.message.includes('403')) {
            provider.failedKeys.add(provider.currentKeyIndex);
            console.warn(`🔑 API key ${provider.currentKeyIndex} for ${providerName} marked as failed`);
        }
    }

    getProviderPriority() {
        // Prioritize providers based on availability and cost
        const priority = [];

        if (this.providers.openrouter.enabled) priority.push('openrouter');
        if (this.providers.gemini.enabled) priority.push('gemini');
        if (this.providers.deepseek.enabled) priority.push('deepseek');
        if (this.providers.anthropic.enabled) priority.push('anthropic');

        return priority;
    }

    getStatus() {
        const status = {
            currentProvider: this.currentProvider,
            requestCount: this.requestCount,
            successCount: this.successCount,
            errorCount: this.errorCount,
            successRate: this.requestCount > 0 ? (this.successCount / this.requestCount * 100).toFixed(1) + '%' : '0%',
            providers: {}
        };

        for (const [name, provider] of Object.entries(this.providers)) {
            status.providers[name] = {
                enabled: provider.enabled,
                apiKeyCount: provider.apiKeys.length,
                failedKeyCount: provider.failedKeys.size,
                availableKeyCount: provider.apiKeys.length - provider.failedKeys.size,
                models: provider.models.length,
                defaultModel: provider.defaultModel
            };
        }

        return status;
    }

    resetFailedProviders() {
        for (const provider of Object.values(this.providers)) {
            provider.failedKeys.clear();
            provider.currentKeyIndex = 0;
        }
        
        console.log('🔄 All failed providers and API keys have been reset');
    }

    getAvailableModels(providerName = null) {
        if (providerName) {
            return this.providers[providerName]?.models || [];
        }

        const allModels = {};
        for (const [name, provider] of Object.entries(this.providers)) {
            if (provider.enabled) {
                allModels[name] = provider.models;
            }
        }

        return allModels;
    }

    /**
     * Multi-Model Voting System
     * Multiple AI models evaluate and vote on the best approach
     */
    async makeVotingRequest(prompt, options = {}) {
        const {
            votingModels = this.getDefaultVotingModels(),
            maxTokens = 1000,
            temperature = 0.7,
            consensusThreshold = 0.6,
            enableDebugVoting = false
        } = options;

        console.log(`🗳️ Starting multi-model voting with ${votingModels.length} models...`);

        // Phase 1: Generate responses from multiple models
        const responses = await this.generateMultipleResponses(prompt, votingModels, {
            maxTokens,
            temperature
        });

        // Phase 2: Have models vote on the best approach
        const votes = await this.conductVoting(prompt, responses, votingModels);

        // Phase 3: Determine consensus and select best response
        const consensus = this.calculateConsensus(votes, consensusThreshold);

        if (enableDebugVoting) {
            console.log('🔍 Voting Debug Info:', JSON.stringify({
                responses: responses.map(r => ({ model: r.model, preview: r.content.substring(0, 100) })),
                votes,
                consensus
            }, null, 2));
        }

        console.log(`✅ Voting complete: ${consensus.winnerModel} selected with ${consensus.confidence}% confidence`);

        return {
            content: consensus.selectedResponse.content,
            provider: consensus.selectedResponse.provider,
            model: consensus.winnerModel,
            responseTime: consensus.selectedResponse.responseTime,
            usage: consensus.selectedResponse.usage,
            votingResults: {
                totalVotes: votes.length,
                consensus: consensus.confidence,
                allResponses: responses.length,
                votingModels: votingModels
            }
        };
    }

    async generateMultipleResponses(prompt, models, options) {
        const responses = [];
        const { maxTokens, temperature } = options;

        for (const modelConfig of models) {
            try {
                const response = await this.makeProviderRequest(
                    modelConfig.provider,
                    prompt,
                    {
                        maxTokens,
                        temperature,
                        model: modelConfig.model
                    }
                );

                responses.push({
                    ...response,
                    provider: modelConfig.provider,
                    modelConfig,
                    responseTime: Date.now()
                });

                console.log(`✅ Response generated: ${modelConfig.provider}/${modelConfig.model}`);

            } catch (error) {
                console.warn(`⚠️ Failed to get response from ${modelConfig.provider}/${modelConfig.model}: ${error.message}`);
            }
        }

        return responses;
    }

    async conductVoting(originalPrompt, responses, votingModels) {
        const votes = [];

        // Create voting prompt
        const votingPrompt = this.createVotingPrompt(originalPrompt, responses);

        for (const modelConfig of votingModels) {
            try {
                const voteResponse = await this.makeProviderRequest(
                    modelConfig.provider,
                    votingPrompt,
                    {
                        maxTokens: 500,
                        temperature: 0.3, // Lower temperature for more consistent voting
                        model: modelConfig.model
                    }
                );

                const vote = this.parseVoteResponse(voteResponse.content);
                votes.push({
                    voter: `${modelConfig.provider}/${modelConfig.model}`,
                    vote,
                    reasoning: voteResponse.content,
                    voterSpecialty: modelConfig.specialty || 'general'
                });

                console.log(`🗳️ Vote cast by ${modelConfig.provider}/${modelConfig.model}: Response ${vote.selectedResponse}`);

            } catch (error) {
                console.warn(`⚠️ Voting failed for ${modelConfig.provider}/${modelConfig.model}: ${error.message}`);
            }
        }

        return votes;
    }

    createVotingPrompt(originalPrompt, responses) {
        let votingPrompt = `You are an AI judge evaluating different responses to a prompt. Your task is to vote for the BEST response based on quality, accuracy, helpfulness, and relevance.

ORIGINAL PROMPT:
"${originalPrompt}"

RESPONSES TO EVALUATE:
`;

        responses.forEach((response, index) => {
            votingPrompt += `
RESPONSE ${index + 1} (${response.provider}/${response.modelConfig.model}):
${response.content}

---
`;
        });

        votingPrompt += `
VOTING INSTRUCTIONS:
1. Analyze each response for quality, accuracy, helpfulness, and relevance
2. Consider the specific strengths of each response
3. Vote for the BEST response by responding with EXACTLY this format:

VOTE: [number]
CONFIDENCE: [1-10]
REASONING: [brief explanation of why this response is best]

Where [number] is the response number (1-${responses.length}) you think is best.
Be objective and focus on which response would be most helpful to the user.`;

        return votingPrompt;
    }

    parseVoteResponse(voteContent) {
        const voteMatch = voteContent.match(/VOTE:\s*(\d+)/i);
        const confidenceMatch = voteContent.match(/CONFIDENCE:\s*(\d+)/i);
        const reasoningMatch = voteContent.match(/REASONING:\s*(.+?)(?:\n|$)/i);

        return {
            selectedResponse: voteMatch ? parseInt(voteMatch[1]) : 1,
            confidence: confidenceMatch ? parseInt(confidenceMatch[1]) : 5,
            reasoning: reasoningMatch ? reasoningMatch[1].trim() : 'No reasoning provided'
        };
    }

    calculateConsensus(votes, threshold) {
        if (votes.length === 0) {
            throw new Error('No votes received for consensus calculation');
        }

        // Count votes for each response
        const voteCounts = {};
        let totalVotes = 0;

        votes.forEach(vote => {
            const responseNum = vote.vote.selectedResponse;
            voteCounts[responseNum] = (voteCounts[responseNum] || 0) + 1;
            totalVotes++;
        });

        // Find winner
        const sortedVotes = Object.entries(voteCounts)
            .map(([responseNum, count]) => ({
                responseNum: parseInt(responseNum),
                count,
                percentage: (count / totalVotes) * 100
            }))
            .sort((a, b) => b.count - a.count);

        const winner = sortedVotes[0];
        const confidence = winner.percentage;

        // Check if consensus threshold is met
        const hasConsensus = confidence >= (threshold * 100);

        return {
            winnerResponseNum: winner.responseNum,
            winnerModel: `Response ${winner.responseNum}`,
            confidence: Math.round(confidence),
            hasConsensus,
            voteCounts,
            allVotes: votes,
            selectedResponse: null // Will be set by caller
        };
    }

    getDefaultVotingModels() {
        const defaultModels = [];

        // Add diverse models for voting
        if (this.providers.openrouter.enabled) {
            defaultModels.push(
                { provider: 'openrouter', model: 'deepseek/deepseek-r1-0528:free', specialty: 'reasoning' },
                { provider: 'openrouter', model: 'mistralai/devstral-small:free', specialty: 'coding' },
                { provider: 'openrouter', model: 'microsoft/phi-4-reasoning:free', specialty: 'analysis' }
            );
        }

        if (this.providers.anthropic.enabled) {
            defaultModels.push(
                { provider: 'anthropic', model: 'claude-3-5-sonnet-20241022', specialty: 'general' }
            );
        }

        if (this.providers.deepseek.enabled) {
            defaultModels.push(
                { provider: 'deepseek', model: 'deepseek-chat', specialty: 'conversation' }
            );
        }

        if (this.providers.gemini.enabled) {
            defaultModels.push(
                { provider: 'gemini', model: 'gemini-2.0-flash-exp', specialty: 'multimodal' }
            );
        }

        return defaultModels.slice(0, 6); // Limit to 6 voting models for efficiency
    }
}
